==> ide-192.7142.36-sources.jar: com/intellij/ui/components/ZoomingDelegate.java <==
/*
 * Copyright 2000-2013 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * @author max
 */
package com.intellij.ui.components;

import com.intellij.util.ui.UIUtil;
import org.jetbrains.annotations.NotNull;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;

public class ZoomingDelegate {
  private final JComponent myContentComponent;
  private final JComponent myViewportComponent;

  private BufferedImage myCachedImage;
  private Point myMagnificationPoint;
  private double myMagnification;

  public ZoomingDelegate(JComponent contentComponent, JComponent viewportComponent) {
    myContentComponent = contentComponent;
    myViewportComponent = viewportComponent;
  }

  public void paint(Graphics g) {
    if (myCachedImage != null && myMagnificationPoint != null) {
      double scale = magnificationToScale(myMagnification);
      int xOffset = (int)(myMagnificationPoint.x - myMagnificationPoint.x * scale);
      int yOffset = (int)(myMagnificationPoint.y - myMagnificationPoint.y * scale);

      Rectangle clip = g.getClipBounds();

      g.setColor(myContentComponent.getBackground());
      g.fillRect(clip.x, clip.y, clip.width, clip.height);

      Graphics2D translated = (Graphics2D)g.create();
      translated.translate(xOffset, yOffset);
      translated.scale(scale, scale);

      UIUtil.drawImage(translated, myCachedImage, 0, 0, null);
    }
  }

  public void magnificationStarted(@NotNull Point at) {
    myMagnificationPoint = at;
  }

  public void magnificationFinished(double magnification) {
    Magnificator magnificator = ((ZoomableViewport)myViewportComponent).getMagnificator();

    if (magnificator != null && Double.compare(magnification, 0) != 0) {
      Point inContent = convertToContentCoordinates(myMagnificationPoint);

      final Point inContentScaled = magnificator.magnify(magnificationToScale(magnification), inContent);

      int vOffset = inContentScaled.y - myMagnificationPoint.y;
      int hOffset = inContentScaled.x - myMagnificationPoint.x;
      myViewportComponent.repaint();
      myViewportComponent.validate();

      scrollTo(vOffset, hOffset);
    }

    myMagnificationPoint = null;
    myMagnification = 0;
    myCachedImage = null;
  }

  protected void scrollTo(int vOffset, int hOffset) {
    JScrollPane pane = JBScrollPane.findScrollPane(myViewportComponent);
    JScrollBar vsb = pane.getVerticalScrollBar();
    vsb.setValue(vOffset);
    JScrollBar hsb = pane.getHorizontalScrollBar();
    hsb.setValue(hOffset);
  }
  
  protected Point convertToContentCoordinates(Point point) {
    return SwingUtilities.convertPoint(myViewportComponent, point, myContentComponent);
  }

  public boolean isActive() {
    return myCachedImage != null;
  }

  private static double magnificationToScale(double magnification) {
    return magnification < 0 ? 1f / (1 - magnification) : (1 + magnification);
  }

  public void magnify(double magnification) {
    double prev = myMagnification;
    myMagnification = magnification;

    if (myCachedImage == null) {
      Rectangle bounds = myViewportComponent.getBounds();
      if (bounds.width <= 0 || bounds.height <= 0) return;

      BufferedImage image = UIUtil.createImage(myViewportComponent.getGraphics(), bounds.width, bounds.height, BufferedImage.TYPE_INT_RGB);

      Graphics graphics = image.getGraphics();
      graphics.setClip(0, 0, bounds.width, bounds.height);
      myViewportComponent.paint(graphics);

      myCachedImage = image;
    }
    if (Double.compare(prev, magnification) != 0) {
      myViewportComponent.repaint();
    }
  }
}

==> ide-193.6015.41-sources.jar: com/intellij/ui/components/ZoomingDelegate.java <==
// Copyright 2000-2019 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.

/*
 * @author max
 */
package com.intellij.ui.components;

import com.intellij.util.ui.ImageUtil;
import com.intellij.util.ui.UIUtil;
import org.jetbrains.annotations.NotNull;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;

public class ZoomingDelegate {
  private final JComponent myContentComponent;
  private final JComponent myViewportComponent;

  private BufferedImage myCachedImage;
  private Point myMagnificationPoint;
  private double myMagnification;

  public ZoomingDelegate(JComponent contentComponent, JComponent viewportComponent) {
    myContentComponent = contentComponent;
    myViewportComponent = viewportComponent;
  }

  public void paint(Graphics g) {
    if (myCachedImage != null && myMagnificationPoint != null) {
      double scale = magnificationToScale(myMagnification);
      int xOffset = (int)(myMagnificationPoint.x - myMagnificationPoint.x * scale);
      int yOffset = (int)(myMagnificationPoint.y - myMagnificationPoint.y * scale);

      Rectangle clip = g.getClipBounds();

      g.setColor(myContentComponent.getBackground());
      g.fillRect(clip.x, clip.y, clip.width, clip.height);

      Graphics2D translated = (Graphics2D)g.create();
      translated.translate(xOffset, yOffset);
      translated.scale(scale, scale);

      UIUtil.drawImage(translated, myCachedImage, 0, 0, null);
    }
  }

  public void magnificationStarted(@NotNull Point at) {
    myMagnificationPoint = at;
  }

  public void magnificationFinished(double magnification) {
    Magnificator magnificator = ((ZoomableViewport)myViewportComponent).getMagnificator();

    if (magnificator != null && Double.compare(magnification, 0) != 0) {
      Point inContent = convertToContentCoordinates(myMagnificationPoint);

      final Point inContentScaled = magnificator.magnify(magnificationToScale(magnification), inContent);

      int vOffset = inContentScaled.y - myMagnificationPoint.y;
      int hOffset = inContentScaled.x - myMagnificationPoint.x;
      myViewportComponent.repaint();
      myViewportComponent.validate();

      scrollTo(vOffset, hOffset);
    }

    myMagnificationPoint = null;
    myMagnification = 0;
    myCachedImage = null;
  }

  protected void scrollTo(int vOffset, int hOffset) {
    JScrollPane pane = JBScrollPane.findScrollPane(myViewportComponent);
    JScrollBar vsb = pane.getVerticalScrollBar();
    vsb.setValue(vOffset);
    JScrollBar hsb = pane.getHorizontalScrollBar();
    hsb.setValue(hOffset);
  }

  protected Point convertToContentCoordinates(Point point) {
    return SwingUtilities.convertPoint(myViewportComponent, point, myContentComponent);
  }

  public boolean isActive() {
    return myCachedImage != null;
  }

  private static double magnificationToScale(double magnification) {
    return magnification < 0 ? 1f / (1 - magnification) : (1 + magnification);
  }

  public void magnify(double magnification) {
    double prev = myMagnification;
    myMagnification = magnification;

    if (myCachedImage == null) {
      Rectangle bounds = myViewportComponent.getBounds();
      if (bounds.width <= 0 || bounds.height <= 0) return;

      BufferedImage image =
        ImageUtil.createImage(myViewportComponent.getGraphics(), bounds.width, bounds.height, BufferedImage.TYPE_INT_RGB);

      Graphics graphics = image.getGraphics();
      graphics.setClip(0, 0, bounds.width, bounds.height);
      myViewportComponent.paint(graphics);

      myCachedImage = image;
    }
    if (Double.compare(prev, magnification) != 0) {
      myViewportComponent.repaint();
    }
  }
}
